require 'rails/all'

module ExecJS
  module Rails
    class Engine < ::Rails::Engine

      config.execjs_rails = ActiveSupport::OrderedOptions.new
      config.execjs_rails.memoize_context = ::Rails.env.production?
      config.execjs_rails.compile_eagerly = ::Rails.env.production?
      config.execjs_rails.on_error = lambda { |error| raise error }
      config.execjs_rails.server_javascripts = ['server.js']
      config.execjs_rails.has_view_function_name = 'execjs_rails_has_view'
      config.execjs_rails.handler_function_name = 'execjs_rails_handler'
      config.execjs_rails.blacklisted_assigns = []

      config.execjs_rails.view_to_function_options = lambda do |view, options|
        # reject keys in the blacklist
        assigns = view.assigns.reject do |key|
          config.execjs_rails.blacklisted_assigns.include?(key)
        end

        assigns.stringify_keys.merge!({
          controller: view.controller_name,
          action: view.action_name,
          csrf_token: view.form_authenticity_token || nil,
          current_user: view.try(:current_user)
        }).merge!(options.stringify_keys).to_json
      end

      config.execjs_rails.debug_dump_prefix = <<-JS
        /*
        * Unwrap this if node-inspector attempts to put it in an extra module
        * Note: other things that are wrapped in module should work the same.
        *   This only addresses the automatic wrapping of the global scope in an
        *   additional module by node-inspector.
        */
        if (typeof module != 'undefined') {
          module = undefined;
          if (typeof exports != 'undefined') {
            exports = undefined;
          }
          arguments.callee.bind(global)(exports, require, module, __filename, __dirname)
        }
      JS

      initializer "execjs_rails.setup_view_helpers" do
        ActiveSupport.on_load(:action_view) do
          include ViewHelper
        end
      end

      initializer "execjs_rails.setup_controller_concern" do
        ActiveSupport.on_load(:action_controller) do
          include Concern
        end
      end

      initializer "execjs_rails.setup_assets_precompile" do
        config.assets.precompile += config.execjs_rails.server_javascripts
      end

      config.after_initialize do |app|
        app.config.execjs_rails.build_source ||= lambda do
          app.config.execjs_rails.server_javascripts.map do |filename|
            # TODO: I hope there is a way to write this that depends much less on knowledge of Rails internals.
            # Surely there is a terser way to tell Rails to us know that there's a precompiled asset or not?
            precompiled_assets_path = "#{app.root}/public/#{app.config.assets.prefix}"
            # If there is a precompile manifest, directly read the precompiled asset file.
            if manifest = Dir["#{precompiled_assets_path}/.sprockets-manifest-*.json"].first
              filename = ActiveSupport::JSON.decode(File.read(manifest))["assets"][filename]
              File.read("#{precompiled_assets_path}/#{filename}")
            # Otherwise let Sprockets build it.
            else
              puts "[execjs_rails] Requesting `#{filename}` from the asset cache."
              app.assets[filename].to_s
            end
          end.join(";")
        end
        Renderer.setup!(app.config.execjs_rails)
        Renderer.reset! if app.config.execjs_rails.compile_eagerly
      end

    end
  end
end
